package es.uca.xmleye.prefs;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.util.List;

/**
 * Interfaz que debe cumplir todo gestor de preferencias a usar por la
 * aplicación.
 */
public interface PreferencesManager {

	/**
	 * Carga las preferencias almacenadas en la fuente, preparando el acceso a
	 * sus valores.
	 * 
	 * @throws PreferencesException
	 *             No se pudieron cargar los datos.
	 */
	public void cargar() throws PreferencesException;

	/**
	 * Guarda las preferencias de forma persistente.
	 * 
	 * @throws PreferencesException
	 *             No se pudieron guardar los datos.
	 */
	public void guardar() throws PreferencesException;

	/**
	 * Cambia el valor de una preferencia escalar. Dicho cambio no será
	 * persistente hasta la ejecución de <code>guardar</code>.
	 * 
	 * @param clave
	 *            Clave de preferencias.
	 * @param escalar
	 *            Valor escalar a almacenar.
	 */
	public void setPrefEscalar(String clave, String escalar);

	/**
	 * Accede al valor en memoria de una preferencia escalar.
	 * 
	 * @param clave
	 *            Clave de preferencias.
	 * @return Valor actual de la preferencia escalar con dicha clave, o
	 *         <code>null</code> si no tiene valor establecido aún. Es posible
	 *         también que la clave exista y aún así tenga valor
	 *         <code>null</code>.
	 */
	public String getPrefEscalar(String clave);

	/**
	 * Accede al valor en memoria de una preferencia escalar, asignando de forma
	 * atómica un valor por defecto si es necesario.
	 * 
	 * @param clave
	 *            Clave de preferencias.
	 * @param omision
	 *            Valor por omisión a establecer si no está disponible.
	 * @return Valor actual de la preferencia escalar con dicha clave, o
	 *         <code>null</code> si no tiene valor establecido aún. Es posible
	 *         también que la clave exista y aún así tenga valor
	 *         <code>null</code>.
	 */
	public String getPrefEscalar(String clave, String omision);

	/**
	 * Cambia el valor establecido de una preferencia con formato de lista.
	 * Dicho cambio no será persistente hasta que se invoque a
	 * <code>guardar</code>.
	 * 
	 * @param clave
	 *            Clave de preferencias.
	 * @param lista
	 *            Valor a almacenar bajo esta clave.
	 */
	public void setPrefLista(String clave, List<String> lista);

	/**
	 * Accede al valor de una preferencia con formato de lista.
	 * 
	 * @param clave
	 *            Clave de preferencias.
	 * @return Lista almacenada bajo la clave <code>clave</code>, o
	 *         <code>null</code> si dicha preferencia aún no ha sido
	 *         establecida.
	 */
	public List<String> getPrefLista(String clave);

	/**
	 * Accede al valor de una preferencia con formato de lista, asignando de
	 * forma atómica un valor por defecto si es necesario.
	 * 
	 * @param clave
	 *            Clave de preferencias.
	 * @param omision
	 *            Valor por omisión a establecer si no está disponible.
	 * @return Lista almacenada bajo la clave <code>clave</code>, o
	 *         <code>null</code> si dicha preferencia aún no ha sido
	 *         establecida.
	 */
	public List<String> getPrefLista(String clave, List<String> omision);

}
