package es.uca.xmleye.gui.trees;

/*
  Copyright (C) 2005-2008 Antonio García Domínguez
  
  This program is free software; you can redistribute it 
  and/or modify it under the terms of the GNU General Public 
  License as published by the Free Software Foundation; 
  either version 2 of the License, or (at your option) any 
  later version.
  
  This program is distributed in the hope that it will be 
  useful, but WITHOUT ANY WARRANTY; without even the implied 
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
  PURPOSE. See the GNU General Public License for more 
  details.
  
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 59 Temple Place, Suite 330, 
  Boston, MA 02111-1307 USA
*/

import junit.framework.*;
import junit.textui.*;

import org.w3c.dom.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import java.io.*;
import es.uca.xmleye.SearchException;
import es.uca.xmleye.gui.trees.DOMTree;
import es.uca.xmleye.gui.trees.DOMTreeModel;
import es.uca.xmleye.xpath.*;

/** 
    Pruebas de unidad de la clase DOMTree.
*/
public class DOMTreeTest extends TestCase 
{
  private Document _docXML;
  private DOMTreeModel _model;
  private DOMTree _tree;

  public DOMTreeTest(String nombre) 
  {
    super(nombre);
  }
  
  /** 
      Comprueba el funcionamiento de los métodos de navegación
      <code>irNodo</code>, <code>irPadre</code>,
      <code>irPrimerHijo</code>, <code>irSigHermano</code> y
      <code>irAntHermano</code>.
  */
  public void testEstructuraArbol() 
  {
    try {
      for (PreorderIterator iter = new PreorderIterator(_model);
	   iter.hasNext(); ) {
	
	final Element elem = (Element)iter.next();
	if (elem.getNodeType() == Node.DOCUMENT_TYPE_NODE) {
	  // Nos saltamos el elemento DOCTYPE: no tiene ruta XPath
	  continue;
	}

	// Comprobamos irNodo(String) al mismo elem. Las rutas son
	// sobre el documento XML, no sobre la raíz del modelo usado
	// por el árbol: la raíz es el nodo documento, como indica el
	// estándar XPath.
	_tree.irNodo(XPathPathManager.getPath(elem, (Node)_model.getDocumento()));
	assertEqual(elem, _tree.getLastSelectedPathComponent());

	// Sólo debe haber Elements en nuestro recorrido: esto se
	// refleja también en nuestros moldes
	Element padre = (elem.getParentNode() instanceof Element ? (Element)elem.getParentNode() : null);
	int elemIdx = 0;

	// Padre
	if (padre != null) {
	  _tree.irPadre();
	  assertEqual(padre, _tree.getLastSelectedPathComponent());

	  // Establecemos el número de hijos
	  elemIdx = _model.getIndexOfChild(padre, elem);

	  // Volvemos
	  _tree.irNodo(elem);
	  assertEqual(elem, _tree.getLastSelectedPathComponent());
	}
	else {
	  // No tiene padre: esto no debería hacer nada
	  _tree.irPadre();
	  assertEqual(elem, _tree.getLastSelectedPathComponent());
	}

	// Hermano anterior
	if (padre != null && elemIdx > 0) {
	  // Tiene hermano anterior según el modelo
	  final Element hermanoAnt = (Element)_model.getChild(padre, elemIdx - 1);
	  _tree.irAntHermano();
	  assertEqual(hermanoAnt, _tree.getLastSelectedPathComponent());
	
	  // Volvemos y comprobamos
	  _tree.irNodo(elem);
	  assertEqual(elem, _tree.getLastSelectedPathComponent());
	}
	else {
	  // No tiene hermano anterior: esto no debería hacer nada
	  _tree.irAntHermano();
	  assertEqual(elem, _tree.getLastSelectedPathComponent());
	}

	// Hermano siguiente
	if (padre != null && elemIdx < _model.getChildCount(padre)-1) {
	  // Tiene hermano siguiente según el _model
	  final Element hermanoSig = (Element)_model.getChild(padre, elemIdx + 1);
	  _tree.irSigHermano();
	  assertEqual(hermanoSig, _tree.getLastSelectedPathComponent());
	
	  // Volvemos y comprobamos
	  _tree.irNodo(elem);
	  assertEqual(elem, _tree.getLastSelectedPathComponent());
	}
	else {
	  // No tiene hermano siguiente: esto no debería hacer nada
	  _tree.irSigHermano();
	  assertEqual(elem, _tree.getLastSelectedPathComponent());
	}

      }
    } catch (SearchException busqEx) {
      fail(busqEx.toString());
    }
  }

  /** Crea el conjunto de casos de prueba para JUnit. */
  public static Test suite() 
  {
    return new TestSuite(DOMTreeTest.class);
  }

  /** Ejecuta las pruebas mediante JUnit. */
  public static void main(String[] args) 
  {
    TestRunner.run(suite());
  }

  /** Preparación: abre un documento XML ya existente y crea un árbol
      con un modelo basado en dicho documento. */
  protected void setUp() throws ParserConfigurationException, SAXException
  {
    try {
      DocumentBuilder dBuilder = 
	DocumentBuilderFactory.newInstance().newDocumentBuilder();

      final FileInputStream fIS = 
	new FileInputStream("build/es/uca/xmleye/gui/trees/testTree.xml");
      _docXML  = dBuilder.parse(fIS);
      _model = new DOMTreeModel(_docXML);
      _tree = new DOMTree(_model);
      
    } catch (FileNotFoundException fEx) {
      System.err.println("No se encontro el fichero.");
    } catch (IOException ioEx) {
      System.err.println("Error de E/S");
    }
  }

  /** Realiza la limpieza tras las pruebas. */
  protected void tearDown() {
    _model = null;
  }

  /** 
      Método de conveniencia para factorizar la comprobación de
      igualdad de elementos.
      @param a Primer elemento.
      @param b Segundo elemento.
  */
  private void assertEqual(Object a, Object b) {
    final Element eA = (Element)a;
    final Element eB = (Element)b;

    if ((eA == null) != (eB == null) 
	|| eA != null && !eA.equals(eB) ) {
      fail("Se esperaba: " + eA
           + ", se obtuvo: " + eB);
    }
  }

}
